/*
 Wireless Gameboy Controller 2 Player Pong Game Test
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 23/09/2018
 Last Modified: 5/11/2018
 
 */

#include <stdio.h>
#include <gb/gb.h>
#include <gb/console.h>
#include <gb/drawing.h>
#include <rand.h>

void init();
WORD receivedata(WORD);
WORD senddata(WORD);

UWORD refreshScore = 1;
UWORD player1Score = 0;
UWORD player2Score = 0;

WORD player1x = 8;
WORD player1y = 90;

WORD player2x = 160;
WORD player2y = 90;

UBYTE keypressData = 0;

WORD ballx = 90;
WORD bally = 90;
UWORD ballSpeed = 5;
WORD ballAngle = 0;
UWORD ballMovement = 0;

UBYTE counter = 0;
UWORD checkPackets = 0;
UBYTE serverMode = 0;


unsigned char char1[] = {
  0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
  0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
};

unsigned char ball[] = {
  0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x18,
  0x18,0x18,0x00,0x00,0x00,0x00,0x00,0x00
};
 

void main() {
	init();
	
	SPRITES_8x8;
	set_sprite_data(0, 8, char1);
	set_sprite_tile(0, 0);
	set_sprite_tile(1, 0);
	set_sprite_tile(2, 0);
	set_sprite_data(1, 8, ball);
	set_sprite_tile(3, 1);
	set_sprite_tile(4, 0);
	set_sprite_tile(5, 0);
	set_sprite_tile(6, 0);
	SHOW_SPRITES;
	
	printf("Hold A = Server");
	delay(500);
	
	if (joypad() & J_A) {
		serverMode = 1;
	}
	
	while (1) {
		// Keypress
		keypressData = 0;
		for (counter = 0; counter < 9; counter++) {
			if (joypad() & J_UP) {
				keypressData = 1;
			}
			else if (joypad() & J_DOWN) { 
				keypressData = 2;
			}
			
			delay(5);
		}
		if (serverMode == 1) {
			if (keypressData == 1 && player1y > 30) {
				player1y -= 4;
			}
			else if (keypressData == 2 && player1y < 142) {
				player1y += 4;
			}
		}
		else {
			if (keypressData == 1 && player2y > 30) {
				player2y -= 4;
			}
			else if (keypressData == 2 && player2y < 142) {
				player2y += 4;
			}
		}
		
		// Move playsers and ball
		move_sprite(0, player1x, player1y);
		player1y -= 8;
		move_sprite(1, player1x, player1y);
		player1y += 16;
		move_sprite(2, player1x, player1y);
		player1y -= 8;
		
		move_sprite(4, player2x, player2y);
		player2y -= 8;
		move_sprite(5, player2x, player2y);
		player2y += 16;
		move_sprite(6, player2x, player2y);
		player2y -= 8;
		
		
		// Ball movement
		move_sprite(3, ballx, bally);
		if (ballMovement == 0) {
			ballx -= ballSpeed;
			bally -= ballAngle;
		}
		else if (ballMovement == 1) {
			ballx += ballSpeed;
			bally += ballAngle;
		}
		
		
		// Detect paddle hit
		if (player1x - ballx + 8 >= 0 && (player1y - bally <= 10 && player1y - bally >= -10)) {
			ballMovement = 1;
			if (player1y - bally <= 10 && player1y - bally > 5) {
				ballAngle -= 2;
			}
			if (player1y - bally <= 5 && player1y - bally > 1) {
				ballAngle -= 1;
			}
			if (player1y - bally >= -10 && player1y - bally < -5) {
				ballAngle += 2;
			}
			if (player1y - bally >= -5 && player1y - bally < -1) {
				ballAngle += 1;
			}
		}
		if (player2x - ballx - 8 <= 0 && (player2y - bally <= 10 && player2y - bally >= -10)) {
			ballMovement = 0;
			if (player2y - bally <= 10 && player2y - bally > 5) {
				ballAngle -= 2;
			}
			if (player2y - bally <= 5 && player2y - bally > 1) {
				ballAngle -= 1;
			}
			if (player2y - bally >= -10 && player2y - bally < -5) {
				ballAngle += 2;
			}
			if (player2y - bally >= -5 && player2y - bally < -1) {
				ballAngle += 1;
			}
		}
		
		// Detect side hits
		if (bally <= 20) {
			ballAngle = ballAngle - (ballAngle * 2);
		}
		if (bally >= 150) {
			ballAngle = ballAngle - (ballAngle * 2);
		}
		
		// Detect misses
		if (ballx == 0) {
			refreshScore = 1;
			ballx = 90;
			bally = 90;
			ballAngle = 0;
			player2Score++;
		}
		if (ballx >= 170) {
			refreshScore = 1;
			ballx = 90;
			bally = 90;
			ballAngle = 0;
			player1Score++;
		}
		
		
		if (refreshScore == 1) {
			gotoxy(0,0);
			printf("                    ");
			gotoxy(0,0);
			printf(" %d                %d", player1Score, player2Score);
			refreshScore = 0;
		}
		
		// If we are the server so we're tracking the ball
		if (serverMode == 1) {
			senddata(4); // 4 writes
			senddata(player1x);
			senddata(player1y);
			senddata(ballx);
			senddata(bally);
			
			checkPackets = receivedata(4); // We want to receive 4 bytes (has to match the send bytes for now), first check if 4 bytes have been received
			if (checkPackets == 4) { // If there is data
				player2x = receivedata(0);
				player2y = receivedata(0);
				receivedata(0);
				receivedata(0);
			}
		}
		else {
			senddata(4); // 4 writes
			senddata(player2x);
			senddata(player2y);
			senddata(0);
			senddata(0);
			
			checkPackets = receivedata(4); // We want to receive 4 bytes (has to match the send bytes for now), first check if 4 bytes have been received
			if (checkPackets == 4) { // If there is data
				player1x = receivedata(0);
				player1y = receivedata(0);
				receivedata(0);
				receivedata(0);
			}
		}
		
		wait_vbl_done();
	}
}

void init() {
	DISPLAY_ON;		// Turn on the display
	NR52_REG = 0x8F;	// Turn on the sound
	NR51_REG = 0x11;	// Enable the sound channels
	NR50_REG = 0x77;	// Increase the volume to its max
}
